#include "glui.h"
#include "stdinc.h"
#include <GL/glut.h>


/*** This object must be used to create a GLUI ***/

GLUI_Master_Object GLUI_Master;


/************************************************ GLUI::GLUI() **********/

int GLUI::init( char *text, int flags, int x, int y ) 
{
  int old_glut_window;

  this->flags = flags;

  strncpy( window_name, text, sizeof(GLUI_String));

  if ( NOT TEST_BIT( flags, GLUI_SAME_WINDOW_BIT ) ) {
    old_glut_window = glutGetWindow();

    create_standalone_window( window_name, x, y );
    setup_default_glut_callbacks();

    if ( old_glut_window > 0 )
      glutSetWindow( old_glut_window );
  }

  return true;
}


/**************************** GLUI_Main::create_standalone_window() ********/

void    GLUI_Main::create_standalone_window( char *name, int x, int y )
{
  glutInitWindowSize( 100, 100 );
  if ( x >= 0 OR y >= 0 )
    glutInitWindowPosition( x, y );
  glutInitDisplayMode( GLUT_RGB ); // | GLUT_DOUBLE );
  glut_window_id = glutCreateWindow( name );
  glDisable( GL_DEPTH_TEST );
}


/**************************** GLUI_Main::setup_default_glut_callbacks() *****/

void   GLUI_Main::setup_default_glut_callbacks( void )
{
  glutDisplayFunc( glui_display_func );
  glutReshapeFunc( glui_reshape_func );
  glutKeyboardFunc( glui_keyboard_func );
  glutSpecialFunc( glui_special_func );
  glutMouseFunc( glui_mouse_func );
  glutMotionFunc( glui_motion_func );
  glutPassiveMotionFunc( glui_passive_motion_func );
  glutEntryFunc( glui_entry_func );
  glutVisibilityFunc( glui_visibility_func );
  //  glutIdleFunc( glui_idle_func );
}


/********************************************** glui_display_func() ********/

void glui_display_func( void )
{
  GLUI *glui;

  //  printf( "display func\n" );

  glui = GLUI_Master.find_glui_by_window_id( glutGetWindow() );

  if ( glui ) {
    glui->display();
  }
}


/********************************************** glui_reshape_func() ********/

void glui_reshape_func( int w, int h )
{
  GLUI *glui;

  glui = GLUI_Master.find_glui_by_window_id( glutGetWindow() );

  if ( glui ) {
    glui->reshape(w,h);
  }
}


/********************************************** glui_keyboard_func() ********/

void glui_keyboard_func(unsigned char key, int x, int y)
{
  GLUI *glui;

  glui = GLUI_Master.find_glui_by_window_id( glutGetWindow() );

  if ( glui ) {
    glui->keyboard(key,x,y);
  }
}


/************************************************ glui_special_func() ********/

void glui_special_func(int key, int x, int y)
{
  GLUI *glui;

   glui = GLUI_Master.find_glui_by_window_id( glutGetWindow() );

  if ( glui ) {
    glui->special(key,x,y);
  }
}



/********************************************** glui_mouse_func() ********/

void glui_mouse_func(int button, int state, int x, int y)
{
  GLUI *glui;

  glui = GLUI_Master.find_glui_by_window_id( glutGetWindow() );

  if ( glui ) {
    /*    printf( "GLUI: %s     id: %d\n", glui->window_name, 
	    glui->get_glut_window_id());*/
    glui->mouse( button, state, x, y );
  }
}


/********************************************** glui_motion_func() ********/

void glui_motion_func(int x, int y)
{
  GLUI *glui;

  glui = GLUI_Master.find_glui_by_window_id( glutGetWindow() );

  if ( glui ) {
    glui->motion(x,y);
  }

}


/**************************************** glui_passive_motion_func() ********/

void glui_passive_motion_func(int x, int y)
{
  GLUI *glui;

  glui = GLUI_Master.find_glui_by_window_id( glutGetWindow() );

  if ( glui ) {
    glui->passive_motion(x,y);
  }
}


/********************************************** glui_entry_func() ********/

void glui_entry_func(int state)
{
  GLUI *glui;

  glui = GLUI_Master.find_glui_by_window_id( glutGetWindow() );

  if ( glui ) {
    glui->entry(state);
  }
}


/******************************************** glui_visibility_func() ********/

void glui_visibility_func(int state)
{
  GLUI *glui;

  //  printf( "IN GLUI VISIBILITY()\n" );
  //  fflush( stdout );

  glui = GLUI_Master.find_glui_by_window_id( glutGetWindow() );

  if ( glui ) {
    glui->visibility(state);
  }
}


/********************************************** glui_idle_func() ********/
/* Send idle event to each glui, then to the main window                */

void glui_idle_func(void)
{
  GLUI *glui;

  glui = (GLUI*) GLUI_Master.gluis.first_child();
  while( glui ) {
    glui->idle();
    
    glui = (GLUI*) glui->next();
  }

  if ( GLUI_Master.glut_idle_CB ) {
    /*** We set the current glut window before calling the user's
      idle function, even though glut explicitly says the window id is 
      undefined in an idle callback.  ***/
    
    /** Check what the current window is first ***/

    /*** Arbitrarily set the window id to the main gfx window of the 
      first glui window ***/
    //   int current_window, new_window;
    //   current_window = glutGetWindow();
    //   if (GLUI_Master.gluis.first_child() != NULL ) {
    //      new_window = ((GLUI_Main*)GLUI_Master.gluis.first_child())->
    //   main_gfx_window_id;

    //   if ( new_window > 0 AND new_window != old_window ) {
    //   /** Window is changed only if its not already the current window **/
    //  glutSetWindow( new_window );
    // }
    //}

    GLUI_Master.glut_idle_CB();
  }
}


/*********************************** GLUI_Master_Object::create_glui() ******/

GLUI    *GLUI_Master_Object::create_glui( char *name, int flags, int x, int y )
{
  GLUI *new_glui;

  new_glui = new GLUI;

  if ( new_glui ) {
    new_glui->init( name, flags, x, y );
    new_glui->link_this_to_parent_last( &this->gluis );
    return new_glui;
  }
  else {
    return NULL;
  }
}


/********************** GLUI_Master_Object::find_glui_by_window_id() ********/

GLUI  *GLUI_Master_Object::find_glui_by_window_id( int window_id )
{
  GLUI_Node *node;

  node = gluis.first_child();
  while( node ) {
    if ( ((GLUI*)node)->get_glut_window_id() == window_id ) 
      return (GLUI*) node;
    
    node = node->next();
  }
  return NULL;
}


/******************************************** GLUI_Main::display() **********/

void    GLUI_Main::display( void )
{
  int       win_w, win_h;

  glClearColor( (float) bkgd_color.r / 255.0,
		(float) bkgd_color.g / 255.0,
		(float) bkgd_color.b / 255.0,
		1.0 );
  glClear( GL_COLOR_BUFFER_BIT ); // | GL_DEPTH_BUFFER_BIT );

  //  glutSwapBuffers(); //performs flush also  // %%%%%%%%%%
  //  return;

  win_w = glutGet( GLUT_WINDOW_WIDTH );
  win_h = glutGet( GLUT_WINDOW_HEIGHT );

  /*** Check here if the window needs resizing ***/
  if ( win_w != main_panel->w OR win_h != main_panel->h ) {
    glutReshapeWindow( main_panel->w, main_panel->h );
    return;
  }

  glMatrixMode( GL_PROJECTION );
  glLoadIdentity();
  gluOrtho2D( 0.0, (float) win_w, 0.0, (float) win_h );

  //  printf( "ORTHO: %f,%f\n", (float) win_w, (float) win_h );
  
  glMatrixMode( GL_MODELVIEW );
  glLoadIdentity();

  /*** Rotate image so y increases upwards, contrary to OpenGL axes ***/
  glTranslatef( (float) win_w/2.0, (float) win_h/2.0, 0.0 );
  glRotatef( 180.0, 0.0, 1.0, 0.0 );
  glRotatef( 180.0, 0.0, 0.0, 1.0 );
  glTranslatef( (float) -win_w/2.0, (float) -win_h/2.0, 0.0 );

  glDrawBuffer( GL_BACK );

  main_panel->draw_recursive( 0, 0 );

  glutSwapBuffers(); //performs flush also  
}




/*************************************** _glutBitmapWidthString() **********/

int _glutBitmapWidthString( void *font, char *s )
{
  char *p = s;
  int  width = 0;

  while( *p != '\0' )  {
    width += glutBitmapWidth( font, *p );
    p++;
  }

  return width;
}

/************************************ _glutBitmapString *********************/
/* Displays the contents of a string using GLUT's bitmap character function */
/* Does not handle newlines                                                 */

void _glutBitmapString( void *font, char *s )
{
  char *p = s;

  while( *p != '\0' )  {
    glutBitmapCharacter( font, *p );
    p++;
  }
}



/****************************** GLUI_Main::reshape() **************/

void    GLUI_Main::reshape( int reshape_w, int reshape_h )
{
  int new_w, new_h;

  pack_controls();

  new_w = main_panel->w;// + 1;
  new_h = main_panel->h;// + 1;

  if ( reshape_w != new_w OR reshape_h != new_h ) {
    this->w = new_w;
    this->h = new_h;
    
    glutReshapeWindow( new_w, new_h );
  }
  else {
  }

  //  if ( NOT TEST_BIT( this->flags, GLUI_SAME_WINDOW_BIT ) )
  
  glViewport( 0, 0, new_w, new_h );

  glutPostRedisplay();
}


/****************************** GLUI_Main::keyboard() **************/

void    GLUI_Main::keyboard(unsigned char key, int x, int y)
{
  GLUI_Control *new_control;

  curr_modifiers = glutGetModifiers();

  /*** If it's a tab or shift tab, we don't pass it on to the controls.
    Instead, we use it to cycle through active controls ***/
  if ( key == '\t' AND mouse_button_down == false ) {
    if ( curr_modifiers & GLUT_ACTIVE_SHIFT ) {
      new_control = find_prev_control( active_control );
    }
    else {
      new_control = find_next_control( active_control );
    }

    /*    if ( new_control )
      printf( "new_control: %s\n", new_control->name );
      */

    disactivate_current_control();
    activate_control( new_control, GLUI_ACTIVATE_TAB );
  }
  else if ( key == ' ' AND active_control 
	    AND active_control->spacebar_mouse_click == true ) { 
    /*** If the user presses the spacebar, and a non-edittext control
      is active, we send it a mouse down event followed by a mouse up
      event (simulated mouse-click) ***/
    
    active_control->mouse_down_handler( 0, 0 );
    active_control->mouse_up_handler( 0, 0, true );
  } else {
    /*** Pass the keystroke onto the active control, if any ***/
    if ( active_control != NULL )
      active_control->key_handler( key, curr_modifiers );
  }
}


/****************************** GLUI_Main::special() **************/

void    GLUI_Main::special(int key, int x, int y)
{
  curr_modifiers = glutGetModifiers();

  /*** Pass the keystroke onto the active control, if any ***/
  if ( active_control != NULL )
    active_control->special_handler( key, glutGetModifiers() );
}



/****************************** GLUI_Main::mouse() **************/

void    GLUI_Main::mouse(int button, int state, int x, int y)
{
  int           callthrough;
  GLUI_Control *control;

  //  printf( "MOUSE: %d %d\n", button, state );

  callthrough = true;

  curr_modifiers = glutGetModifiers();

  if ( button == GLUT_LEFT ) {
    control = find_control( x, y );

    //    printf( "control: %s\n", control->name );
    
    if ( mouse_button_down AND active_control != NULL AND
	 state == GLUT_UP ) {
      /** We just released the mouse, which was depressed at some
	control **/

      callthrough = active_control->
	mouse_up_handler( x, y, control==active_control);

      if ( active_control->active_type == GLUI_CONTROL_ACTIVE_MOUSEDOWN AND 0){
	/*** This is a control that needs to be disactivated when the
	  mouse button is released ****/
	disactivate_current_control();
      }
    }
    else {
      if ( control ) {
	if ( NOT mouse_button_down AND state == GLUT_DOWN ) {
	  /*** We just pressed the mouse down at some control ***/

	  if ( active_control != control ) {
	    if ( active_control != NULL ) {
	      /** There is an active control still - disactivate it ***/
	      disactivate_current_control();
	    }
	  }

	  if ( control->enabled ) {
	    activate_control( control, GLUI_ACTIVATE_MOUSE );
	    callthrough    = control->mouse_down_handler( x, y );
	  }
	}
      }
    }

    if ( state == GLUT_DOWN )
      mouse_button_down = true;
    else if ( state == GLUT_UP )
      mouse_button_down = false;
  }

  if ( callthrough AND glut_mouse_CB )
    glut_mouse_CB( button, state, x, y );
}


/****************************** GLUI_Main::motion() **************/

void    GLUI_Main::motion(int x, int y)
{
  int           callthrough;
  GLUI_Control *control;

  //  printf( "MOTION: %d %d\n", x, y );

  callthrough = true;

  control = find_control(x,y);
  
  if ( mouse_button_down AND active_control != NULL ) {
    callthrough = 
      active_control->mouse_held_down_handler(x,y,control==active_control);
  }
  
  if ( callthrough AND glut_motion_CB )
    glut_motion_CB(x,y);
}


/*********************** GLUI_Main::passive_motion() **************/

void    GLUI_Main::passive_motion(int x, int y)
{
  GLUI_Control *control;

  control = find_control( x, y );
  if ( control ) {
    if ( control->type == GLUI_CONTROL_EDITTEXT ) {
      curr_cursor = GLUT_CURSOR_TEXT;
      glutSetCursor( GLUT_CURSOR_TEXT );
      return;
    }
  }

  if ( curr_cursor != GLUT_CURSOR_INHERIT ) {
    curr_cursor = GLUT_CURSOR_INHERIT;
    glutSetCursor( GLUT_CURSOR_INHERIT );    
  }
}


/****************************** GLUI_Main::entry() **************/

void    GLUI_Main::entry(int state)
{
  if ( NOT active_control OR active_control->type == GLUI_CONTROL_EDITTEXT
       OR active_control->type == GLUI_CONTROL_SPINNER )
    glutSetCursor( GLUT_CURSOR_INHERIT );
}


/****************************** GLUI_Main::visibility() **************/

void    GLUI_Main::visibility(int state)
{
}


/****************************** GLUI_Main::idle() **************/

void    GLUI_Main::idle(void)
{
  /*** Pass the idle event onto the active control, if any ***/

  //  printf( "IDLE \t" );

  if ( active_control != NULL ) {
    /* First we check if the control actually needs the idle right now.
       Otherwise, let's avoid wasting cycles and OpenGL context switching */

    if ( active_control->needs_idle() ) {
      /*** Set the current glut window to the glui window */
      /*** But don't change the window if we're already at that window ***/

      if ( glut_window_id > 0 AND glutGetWindow() != glut_window_id ) {
	glutSetWindow( glut_window_id );
      }
      
      active_control->idle();
    }
  }
}


/******************************************* GLUI_Main::find_control() ******/

GLUI_Control  *GLUI_Main::find_control( int x, int y )
{
  GLUI_Control *node, *last_container;

  last_container = NULL;

  node = main_panel;
  while( node != NULL ) {
    if ( PT_IN_BOX( x, y, 
		    node->x_abs, node->x_abs + node->w, 
		    node->y_abs, node->y_abs + node->h )) {
      /*** Point is inside current node ***/
      
      if ( node->first_child() == NULL ) {
	/*** SPECIAL CASE: for edittext boxes, we make sure click is
	  in box, and not on name string.  This should be generalized
	  for all controls later... ***/
	if ( node->type == GLUI_CONTROL_EDITTEXT ) {
	  if ( x < node->x_abs + ((GLUI_EditText*)node)->text_x_offset )
	    return (GLUI_Control*) node->parent();
	}

	return node;   /* point is inside this node, and node has no children,
			  so return this node as the selected node */
      }
      else {
	/*** This is a container class ***/
	last_container = node;
	node = (GLUI_Control*) node->first_child();  /* Descend into child */
      }
      
    }
    else {
      node = (GLUI_Control*) node->next();
    }
  }
 
  /** No leaf-level nodes found to accept the mouse click, so
    return the last container control found which DOES accept the click **/
  
  if ( last_container ) {
    //    printf( "ctrl: '%s'\n", last_container->name );
  
    return last_container;
  }
  else {
    return NULL;
  }
}


/************************************* GLUI_Main::pack_controls() ***********/

void      GLUI_Main::pack_controls( void )
{
  main_panel->pack(0,0);

  /**** Now align controls within their bounds ****/
  align_controls( main_panel );
}


/************************************ GLUI_Main::align_controls() **********/

void    GLUI_Main::align_controls( GLUI_Control *control )
{
  GLUI_Control *child;

  control->align();

  child = (GLUI_Control*) control->first_child();

  while( child != NULL ) {
    align_controls( child );
    
    child = (GLUI_Control*)child->next();
  }  
}



/*********************************** GLUI::set_main_gfx_window() ************/

void   GLUI::set_main_gfx_window( int window_id )
{
  main_gfx_window_id = window_id;
}


/********************************* GLUI_Main::post_update_main_gfx() ********/

void   GLUI_Main::post_update_main_gfx( void )
{
  int old_window;

  if ( main_gfx_window_id > 0 ) {
    old_window = glutGetWindow();
    glutSetWindow( main_gfx_window_id );
    glutPostRedisplay();
    if( old_window > 0 )
      glutSetWindow( old_window );
  }
}


/********************************* GLUI_Main::set_front_draw_buffer() ********/

int          GLUI_Main::set_front_draw_buffer( void )
{
  int state;
  
  glGetIntegerv( GL_DRAW_BUFFER, &state );

  glDrawBuffer( GL_FRONT );
  
  return state;
}
 

/********************************* GLUI_Main::restore_draw_buffer() **********/

void         GLUI_Main::restore_draw_buffer( int buffer_state )
{
  glDrawBuffer( buffer_state );
}


/******************************************** GLUI_Main::GLUI_Main() ********/

GLUI_Main::GLUI_Main( void ) 
{
  glut_display_CB         = NULL;
  glut_reshape_CB         = NULL;
  glut_keyboard_CB        = NULL;
  glut_special_CB         = NULL;
  glut_mouse_CB           = NULL;
  glut_motion_CB          = NULL;
  glut_passive_motion_CB  = NULL;
  glut_entry_CB           = NULL;
  glut_visibility_CB      = NULL;
  
  mouse_button_down       = false;
  w                       = 0;
  h                       = 0;
  active_control          = NULL;
  main_gfx_window_id      = -1;
  glut_window_id          = -1;
  curr_modifiers          = 0;

  font                    = GLUT_BITMAP_HELVETICA_12;
  curr_cursor             = GLUT_CURSOR_INHERIT;

  bkgd_color.set( 200, 200, 200 );

  /*** Create the main panel ***/
  main_panel              = new GLUI_Panel;
  main_panel->set_int_val( GLUI_PANEL_NONE );
  main_panel->glui        = (GLUI*) this;

}

/************************************ GLUI_Main::draw_raised_box() **********/

void      GLUI_Main::draw_raised_box( int w, int h )
{
  glColor3ub( bkgd_color.r, bkgd_color.g, bkgd_color.b );
  glBegin( GL_LINE_LOOP );
  glVertex2i( 1, 1 );         glVertex2i( w-1, 1 );
  glVertex2i( w-1, h-1 );     glVertex2i( 1, h-1 );
  glEnd();


  glColor3d( 1.0, 1.0, 1.0 );
  glBegin( GL_LINE_STRIP );
  glVertex2i( 0, h );
  glVertex2i( 0, 0 );
  glVertex2i( w, 0 );
  glEnd();

  glColor3d( 0.0, 0.0, 0.0 );
  glBegin( GL_LINE_STRIP );
  glVertex2i( w, 0 );
  glVertex2i( w, h );
  glVertex2i( 0, h );
  glEnd();

  glColor3d( .5, .5, .5 );
  glBegin( GL_LINE_STRIP );
  glVertex2i( w-1, 1 );
  glVertex2i( w-1, h-1 );
  glVertex2i( 1, h-1 );
  glEnd();
}


/************************************* GLUI_Main::activate_control() *********/

void         GLUI_Main::activate_control( GLUI_Control *control, int how )
{
  if ( control != NULL AND control->can_activate AND control->enabled ) {
    active_control = control;
    
    control->activate(how);

    if ( NOT active_control->is_container ) {
      active_control->translate_and_draw_front();
    }
  }
  else {
    active_control = NULL;
  }
}


/************************* GLUI_Main::disactivate_current_control() **********/

void         GLUI_Main::disactivate_current_control( void )
{
  int orig;

  if ( active_control != NULL ) {
    orig = active_control->set_to_glut_window();

    active_control->disactivate();
    
    /** If this isn't a container control, then redraw it in its 
      disactivated state.  Container controls, such as panels, look
      the same activated or not **/
    if ( NOT active_control->is_container ) {
      active_control->translate_and_draw_front();
    }

    active_control->restore_window( orig );

    active_control = NULL;
  }
}

/****************************** GLUI_Main::find_next_control() **************/

GLUI_Control  *GLUI_Main::find_next_control_( GLUI_Control *control )
{
  /*** THIS IS NOT find_next_control()!  This is an unused older
    version (look at the underscore at the end) ***/

  if ( control == NULL )
    return find_next_control_rec( main_panel );
  else
    return find_next_control_rec( control );
}

/****************************** GLUI_Main::find_next_control() **************/

GLUI_Control  *GLUI_Main::find_next_control_rec( GLUI_Control *control )
{
  GLUI_Control *child = NULL, *rec_control, *sibling;

  /*** Recursively investigate children ***/
  child = (GLUI_Control*) control->first_child();
  if ( child ) {
    /*** If we can activate the first child, then do so ***/
    if ( child->can_activate AND child->enabled )
      return child;
    else     /*** Recurse into first child ***/
      rec_control = find_next_control_rec( child );    

    if ( rec_control )
      return rec_control;
  }

  /*** At this point, either we don't have children, or the child cannot
    be activated.  So let's try the next sibling ***/

  sibling = (GLUI_Control*) control->next();
  if ( sibling ) {
    if ( sibling->can_activate AND sibling->enabled )
      return sibling;
    else     /*** Recurse into sibling ***/
      rec_control = find_next_control_rec( sibling );    

    if ( rec_control )
      return rec_control;
  }
  
  return NULL;
}


/****************************** GLUI_Main::find_next_control() **************/

GLUI_Control  *GLUI_Main::find_next_control( GLUI_Control *control )
{
  GLUI_Control *tmp_control = NULL;
  int           back_up;
  
  if ( control == NULL )
    control = main_panel;

  while( control != NULL ) {
    /** see if this control has a child **/
    tmp_control = (GLUI_Control*) control->first_child();

    if ( tmp_control != NULL ) {
      if ( tmp_control->can_activate AND tmp_control->enabled )
	return tmp_control;
      
      control = tmp_control;  /* Descend into child */
      continue;
    }
    
    /*** At this point, control has no children ***/

    /** see if this control has a next sibling **/
    tmp_control = (GLUI_Control*) control->next();
    
    if ( tmp_control != NULL ) {
      if ( tmp_control->can_activate AND tmp_control->enabled )
	return tmp_control;
      
      control = tmp_control;  
      continue;
    }

    /** back up until we find a sibling of an ancestor **/
    back_up = true;
    while ( control->parent() AND back_up ) {
      control = (GLUI_Control*) control->parent();

      if ( control->next() ) {
	control = (GLUI_Control*) control->next();  
	if ( control->can_activate AND control->enabled )
	  return control;
	else
	  back_up = false;

	/***	if ( control->is_container ) {
	  tmp_control = control;
	  control     = NULL;
	  break;
	}
	else {
	  back_up = false;
	}
	***/
      }
    }

    /** Check if we've cycled back to the top... if so, return NULL **/
    if ( control == main_panel ) {
      return NULL;
    }
  }
  /*
  if ( tmp_control != NULL AND tmp_control->can_activate AND
       tmp_control->enabled ) {
    return tmp_control;
  }*/

  return NULL;
}


/****************************** GLUI_Main::find_prev_control() **************/

GLUI_Control  *GLUI_Main::find_prev_control( GLUI_Control *control )
{
  GLUI_Control *tmp_control, *next_control;

  if ( control == NULL ) {        /* here we find the last valid control */
    next_control = main_panel;
  
    do {
      tmp_control  = next_control;
      next_control = find_next_control( tmp_control ); 
    } while( next_control != NULL );

    return tmp_control;    
  }
  else {                         /* here we find the actual previous control */
    next_control = main_panel;
  
    do {
      tmp_control  = next_control;
      next_control = find_next_control( tmp_control ); 
    } while( next_control != NULL AND next_control != control );
    
    if ( next_control == NULL OR tmp_control == main_panel )
      return NULL;
    else 
      return tmp_control;
  }
}

/************************* GLUI_Master_Object::set_glutIdleFunc() ***********/

void    GLUI_Master_Object::set_glutIdleFunc(void (*f)(void))
{
  glut_idle_CB = f;
  glutIdleFunc( glui_idle_func );
}


/**************************************** GLUI::disable() ********************/

void   GLUI::disable( void )
{ 
  disactivate_current_control();
  main_panel->disable(); 
}


/******************************************** GLUI::sync_live() **************/

void   GLUI::sync_live( void )
{
  main_panel->sync_live(true);
}
